@@ -46,6 +46,8 @@
 #' Next, the grouping information is replaced with the respective row ids to generate training and test sets.
 #' The sets can be accessed via `$train_set(i)` and `$test_set(i)`, respectively.
 #'
+#' @section Inheriting:
+#' It is possible to overwrite both `private$.get_instance()` to have full control, or only `private$.sample()` when one wants to use the pre-defined mechanism for stratification and grouping.
 #'
 #' @template seealso_resampling
 #' @export
@@ -101,6 +103,10 @@ Resampling = R6Class("Resampling",
     #'   The hash of the [Task] which was passed to `r$instantiate()`.
     task_hash = NA_character_,
 
+    #' @field task_row_hash (`character(1)`)\cr
+    #'   The hash of the row ids of the [Task] which was passed to `r$instantiate()`.
+    task_row_hash = NA_character_,
+
     #' @field task_nrow (`integer(1)`)\cr
     #'   The number of observations of the [Task] which was passed to `r$instantiate()`.
     #'
@@ -141,10 +147,13 @@ Resampling = R6Class("Resampling",
     #' Printer.
     #' @param ... (ignored).
     print = function(...) {
-      catn(format(self), if (is.null(self$label) || is.na(self$label)) "" else paste0(": ", self$label))
-      catn(str_indent("* Iterations:", self$iters))
-      catn(str_indent("* Instantiated:", self$is_instantiated))
-      catn(str_indent("* Parameters:", as_short_string(self$param_set$values, 1000L)))
+      msg_h = if (is.null(self$label) || is.na(self$label)) "" else paste0(": ", self$label)
+      cat_cli({
+        cli_h1("{.cls {class(self)[1L]}} {msg_h}")
+        cli_li("Iterations: {.val {self$iters}}")
+        cli_li("Instantiated: {.val {self$is_instantiated}}")
+        cli_li("Parameters: {as_short_string(self$param_set$values, 1000L)}")
+      })
     },
 
     #' @description
@@ -166,26 +175,10 @@ Resampling = R6Class("Resampling",
     #' the object in its previous state.
     instantiate = function(task) {
       task = assert_task(as_task(task))
-      strata = task$strata
-      groups = task$groups
-
-      if (is.null(strata)) {
-        if (is.null(groups)) {
-          instance = private$.sample(task$row_ids, task = task)
-        } else {
-          private$.groups = groups
-          instance = private$.sample(unique(groups$group), task = task)
-        }
-      } else {
-        if (!is.null(groups)) {
-          stopf("Cannot combine stratification with grouping")
-        }
-        instance = private$.combine(lapply(strata$row_id, private$.sample, task = task))
-      }
-
       private$.hash = NULL
-      self$instance = instance
+      self$instance = private$.get_instance(task)
       self$task_hash = task$hash
+      self$task_row_hash = task$row_hash
       self$task_nrow = task$nrow
       invisible(self)
     },
@@ -253,6 +246,24 @@ Resampling = R6Class("Resampling",
     .hash = NULL,
     .groups = NULL,
 
+    .get_instance = function(task) {
+      strata = task$strata
+      groups = task$groups
+      if (is.null(strata)) {
+        if (is.null(groups)) {
+          private$.sample(task$row_ids, task = task)
+        } else {
+          private$.groups = groups
+          private$.sample(unique(groups$group), task = task)
+        }
+      } else {
+        if (!is.null(groups)) {
+          stopf("Cannot combine stratification with grouping")
+        }
+        private$.combine(lapply(strata$row_id, private$.sample, task = task))
+      }
+    },
+
     .get_set = function(getter, i) {
       if (!self$is_instantiated) {
         stopf("Resampling '%s' has not been instantiated yet", self$id)
