@@ -44,7 +44,7 @@ ResamplingCV = R6Class("ResamplingCV", inherit = Resampling,
       ps = ps(
         folds = p_int(2L, tags = "required")
       )
-      ps$values = list(folds = 10L)
+      ps$set_values(folds = 10L)
 
       super$initialize(id = "cv", param_set = ps,
         label = "Cross-Validation", man = "mlr3::mlr_resamplings_cv")
